@@ -4,14 +4,8 @@ from typing import Dict
 import torch
 from torch.optim.optimizer import Optimizer
 
-from pytorch_optimizer.types import (
-    BETAS,
-    BUFFER,
-    CLOSURE,
-    DEFAULT_PARAMETERS,
-    LOSS,
-    PARAMS,
-)
+from pytorch_optimizer.gc import centralize_gradient
+from pytorch_optimizer.types import BETAS, BUFFER, CLOSURE, DEFAULTS, LOSS, PARAMETERS
 
 
 class Ranger(Optimizer):
@@ -32,7 +26,7 @@ class Ranger(Optimizer):
 
     def __init__(
         self,
-        params: PARAMS,
+        params: PARAMETERS,
         lr: float = 1e-3,
         alpha: float = 0.5,
         k: int = 6,
@@ -44,32 +38,30 @@ class Ranger(Optimizer):
         gc_conv_only: bool = False,
     ):
         """
-        :param params: PARAMS. iterable of parameters to optimize
-            or dicts defining parameter groups
+        :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups
         :param lr: float. learning rate.
-        :param n_sma_threshold: int. (recommended is 5)
-        :param betas: BETAS. coefficients used for computing running averages
-            of gradient and the squared hessian trace
-        :param eps: float. term added to the denominator to improve numerical stability
+        :param betas: BETAS. coefficients used for computing running averages of gradient and the squared hessian trace
         :param weight_decay: float. weight decay (L2 penalty)
+        :param n_sma_threshold: int. (recommended is 5)
         :param use_gc: bool. use Gradient Centralization (both convolution & fc layers)
         :param gc_conv_only: bool. use Gradient Centralization (only convolution layer)
+        :param eps: float. term added to the denominator to improve numerical stability
         """
         self.lr = lr
         self.alpha = alpha
         self.k = k
         self.n_sma_threshold = n_sma_threshold
         self.betas = betas
-        self.eps = eps
         self.weight_decay = weight_decay
         self.use_gc = use_gc
+        self.eps = eps
 
         self.gc_gradient_threshold: int = 3 if gc_conv_only else 1
         self.buffer: BUFFER = [[None, None, None] for _ in range(10)]
 
         self.check_valid_parameters()
 
-        defaults: DEFAULT_PARAMETERS = dict(
+        defaults: DEFAULTS = dict(
             lr=lr,
             alpha=alpha,
             k=k,
@@ -84,16 +76,16 @@ class Ranger(Optimizer):
     def check_valid_parameters(self):
         if self.lr < 0.0:
             raise ValueError(f'Invalid learning rate : {self.lr}')
-        if self.eps < 0.0:
-            raise ValueError(f'Invalid eps : {self.eps}')
+        if self.k < 1:
+            raise ValueError(f'Invalid lookahead step {self.k}')
         if self.weight_decay < 0.0:
             raise ValueError(f'Invalid weight_decay : {self.weight_decay}')
         if not 0.0 <= self.betas[0] < 1.0:
             raise ValueError(f'Invalid beta_0 : {self.betas[0]}')
         if not 0.0 <= self.betas[1] < 1.0:
             raise ValueError(f'Invalid beta_1 : {self.betas[1]}')
-        if self.k < 1:
-            raise ValueError(f'Invalid lookahead step {self.k}')
+        if self.eps < 0.0:
+            raise ValueError(f'Invalid eps : {self.eps}')
 
     def __setstate__(self, state: Dict):
         super().__setstate__(state)
@@ -107,38 +99,27 @@ class Ranger(Optimizer):
                     continue
 
                 grad = p.grad.data.float()
-
                 if grad.is_sparse:
-                    raise RuntimeError(
-                        'Ranger optimizer does not support sparse gradients'
-                    )
+                    raise RuntimeError('Ranger optimizer does not support sparse gradients')
 
                 p_data_fp32 = p.data.float()
 
                 state = self.state[p]
-
                 if len(state) == 0:
                     state['step'] = 0
                     state['exp_avg'] = torch.zeros_like(p_data_fp32)
                     state['exp_avg_sq'] = torch.zeros_like(p_data_fp32)
-
                     state['slow_buffer'] = torch.empty_like(p.data)
                     state['slow_buffer'].copy_(p.data)
                 else:
                     state['exp_avg'] = state['exp_avg'].type_as(p_data_fp32)
-                    state['exp_avg_sq'] = state['exp_avg_sq'].type_as(
-                        p_data_fp32
-                    )
+                    state['exp_avg_sq'] = state['exp_avg_sq'].type_as(p_data_fp32)
 
                 exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
                 beta1, beta2 = group['betas']
 
-                if grad.dim() > self.gc_gradient_threshold:
-                    grad.add_(
-                        -grad.mean(
-                            dim=tuple(range(1, grad.dim())), keepdim=True
-                        )
-                    )
+                if self.use_gc and grad.dim() > self.gc_gradient_threshold:
+                    grad = centralize_gradient(grad, gc_conv_only=False)
 
                 state['step'] += 1
 
@@ -153,9 +134,7 @@ class Ranger(Optimizer):
                     buffered[0] = state['step']
                     beta2_t = beta2 ** state['step']
                     n_sma_max = 2 / (1 - beta2) - 1
-                    n_sma = n_sma_max - 2 * state['step'] * beta2_t / (
-                        1 - beta2_t
-                    )
+                    n_sma = n_sma_max - 2 * state['step'] * beta2_t / (1 - beta2_t)
                     buffered[1] = n_sma
                     if n_sma > self.n_sma_threshold:
                         step_size = math.sqrt(
@@ -169,18 +148,15 @@ class Ranger(Optimizer):
                         ) / (1 - beta1 ** state['step'])
                     else:
                         step_size = 1.0 / (1 - beta1 ** state['step'])
+
                     buffered[2] = step_size
 
                 if group['weight_decay'] != 0:
-                    p_data_fp32.add_(
-                        -group['weight_decay'] * group['lr'], p_data_fp32
-                    )
+                    p_data_fp32.add_(-group['weight_decay'] * group['lr'], p_data_fp32)
 
                 if n_sma > self.n_sma_threshold:
                     denom = exp_avg_sq.sqrt().add_(group['eps'])
-                    p_data_fp32.addcdiv_(
-                        -step_size * group['lr'], exp_avg, denom
-                    )
+                    p_data_fp32.addcdiv_(-step_size * group['lr'], exp_avg, denom)
                 else:
                     p_data_fp32.add_(-step_size * group['lr'], exp_avg)
 
